/*
 *  $Id: asapinstance.h,v 1.23 2002/08/22 09:37:22 dreibh Exp $
 *
 * RSerPool implementation.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * Acknowledgement
 * This work was partially funded by the Bundesministerium fr Bildung und
 * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
 * The authors alone are responsible for the contents.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * There are two mailinglists available at http://www.sctp.de/rserpool.html
 * which should be used for any discussion related to this implementation.
 *
 * Contact: rsplib-discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: ASAP Instance
 *
 */


#ifndef ASAPINSTANCE_H
#define ASAPINSTANCE_H


#include "tdtypes.h"
#include "dispatcher.h"
#include "asapmessage.h"
#include "poolnamespace.h"
#include "asaperror.h"
#include "serverhunter.h"

#include <glib.h>


#ifdef __cplusplus
extern "C" {
#endif



struct ASAPInstance
{
   struct Dispatcher*     StateMachine;

   int                    NameServer;
   struct ServerHunter*   NameServerHunter;

   struct PoolNamespace*  Cache;
   struct Timer*          CacheMaintenanceTimer;
   GList*                 CachePurgeList;



   char*                  AsapConfigFile;

   card64                 ServerHuntTimeout;
   card64                 MaxServerHuntTimeout;
   cardinal               MaxServerHuntTrials;

   card64                 CacheMaintenanceInterval;
   card64                 CacheElementTimeout;
   card64                 CachePoolLookupInterval;

   cardinal               UserConnectionMaxTrials;
   card64                 UserConnectionEstablishmentTimeout;

   cardinal               NameServerMaxRequestTrials;
   card64                 NameServerRequestTimeout;
   card64                 NameServerResponseTimeout;
};



#define ASAP_DEFAULT_CONFIG_FILE                           "./asap.conf"

#define ASAP_DEFAULT_SERVER_HUNT_TIMEOUT                   2000000
#define ASAP_DEFAULT_MAX_SERVER_HUNT_TIMEOUT               5000000
#define ASAP_DEFAULT_MAX_SERVER_HUNT_TRIALS                      3

#define ASAP_DEFAULT_CACHE_MAINTENANCE_INTERVAL            300000000 // ??????
#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT                 200000000 // ??????
#define ASAP_DEFAULT_CACHE_POOL_LOOKUP_INTERVAL            1000000

#define ASAP_DEFAULT_USER_CONNECTION_MAX_TRIALS                 50
#define ASAP_DEFAULT_USER_CONNECTION_ESTABLISHMENT_TIMEOUT 3333333

#define ASAP_DEFAULT_NAMESERVER_MAX_REQUEST_TRIALS               1
#define ASAP_DEFAULT_NAMESERVER_REQUEST_TIMEOUT            4443333
#define ASAP_DEFAULT_NAMESERVER_RESPONSE_TIMEOUT           4444444

#define ASAP_BUFFER_SIZE 65536


/**
  * Constructor.
  *
  * @param dispatcher Dispatcher.
  * @return ASAPInstance or NULL in case of error.
  */
struct ASAPInstance* asapNew(struct Dispatcher* dispatcher);

/**
  * Destructor.
  *
  * @param asap ASAPInstance.
  */
void asapDelete(struct ASAPInstance* asap);

/**
  * Register pool element.
  *
  * @param asap ASAPInstance.
  * @param poolHandle Pool handle.
  * @param poolElement Pool Element.
  * @return ASAP_Okay in case of success; error code otherwise.
  */
enum ASAPError asapRegister(struct ASAPInstance* asap,
                            struct PoolHandle*   poolHandle,
                            struct PoolElement*  poolElement);

/**
  * Deregister pool element.
  *
  * @param asap ASAPInstance.
  * @param poolHandle Pool handle.
  * @param identifier Pool element identifier.
  * @return ASAP_Okay in case of success; error code otherwise.
  */
enum ASAPError asapDeregister(struct ASAPInstance*        asap,
                              struct PoolHandle*          poolHandle,
                              const PoolElementIdentifier identifier);

/**
  * Query name server for pool handle and select one pool element
  * by policy.
  *
  * @param asap ASAPInstance.
  * @param poolHandle Pool handle.
  * @param error Reference to store error code to.
  * @return Pool element or NULL in case of error.
  */
struct PoolElement* asapSelectPoolElement(struct ASAPInstance* asap,
                                          struct PoolHandle*   poolHandle,
                                          enum ASAPError*      error);

/**
  * Report failure of pool element.
  *
  * @param asap ASAPInstance.
  * @param handle Pool element handle.
  */
void asapPoolElementFailure(struct ASAPInstance*    asap,
                            const PoolElementHandle handle);

/**
  * Do name resolution of given pool handle. The resulting pool pointer
  * will be stored to the variable poolPtr.
  *
  * @param asap ASAPInstance.
  * @param poolHandle Pool handle.
  * @param poolPtr Reference to store reply to.
  * @return ASAP_Okay in case of success; error code otherwise.
  */
enum ASAPError asapNameResolution(struct ASAPInstance* asap,
                                  struct PoolHandle*   poolHandle,
                                  struct Pool**        poolPtr);


#ifdef __cplusplus
}
#endif


#endif
